@@ -26,11 +26,26 @@
 #include <config.h>
 #endif
 
+#define _GNU_SOURCE
+
+#include <glib.h>
+#ifdef G_OS_WIN32
+#include <process.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h> /* for MAXHOSTNAMELEN */
+#else
+#define MAXHOSTNAMELEN 64
+#endif
+#include <sys/types.h>
 #include <string.h>
+#include <unistd.h>
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
 #include <ctype.h>
 #include <errno.h>
 
@@ -218,14 +233,18 @@ datetok (const char *date)
 	start = date;
 	while (*start) {
 		/* kill leading whitespace */
-		for ( ; *start && isspace ((int) *start); start++);
+		while (*start && isspace ((int) *start))
+			start++;
 		
-		mask = 0;
+		if (*start == '\0')
+			break;
+		
+		mask = gmime_datetok_table[*start];
 		
 		/* find the end of this token */
-		for (end = start; *end && !strchr ("-/,\t\r\n ", *end); end++) {
-			mask |= gmime_datetok_table[*end];
-		}
+		end = start + 1;
+		while (*end && !strchr ("-/,\t\r\n ", *end))
+			mask |= gmime_datetok_table[*end++];
 		
 		if (end != start) {
 			token = g_malloc (sizeof (struct _date_token));
@@ -744,21 +763,33 @@ g_mime_utils_generate_message_id (const char *fqdn)
 #endif
 	static unsigned int count = 0;
 	char host[MAXHOSTNAMELEN + 1];
+#ifdef HAVE_GETHOSTBYNAME
 	struct hostent *h = NULL;
+#endif
 	char *msgid;
 	
 	if (!fqdn) {
+#ifdef HAVE_GETHOSTNAME
 		if (gethostname (host, sizeof (host)) == 0) {
+#ifdef HAVE_GETHOSTBYNAME
 			h = gethostbyname (host);
+#endif /* HAVE_GETHOSTBYNAME */
 		} else {
 			host[0] = '\0';
 		}
+#else
+		host[0] = '\0';
+#endif /* HAVE_GETHOSTNAME */
 		
+#ifdef HAVE_GETHOSTBYNAME
 		fqdn = h ? h->h_name : (*host ? host : "localhost.localdomain");
+#else
+		fqdn = *host ? host : "localhost.localdomain";
+#endif
 	}
 	
 	MUTEX_LOCK ();
-	msgid = g_strdup_printf ("%ul.%ul.%ul@%s", time (NULL), getpid (), count++, fqdn);
+	msgid = g_strdup_printf ("%ul.%ul.%ul@%s", (unsigned int) time (NULL), getpid (), count++, fqdn);
 	MUTEX_UNLOCK ();
 	
 	return msgid;
@@ -1507,12 +1538,14 @@ rfc2047_encode_word (GString *string, const unsigned char *word, size_t len,
 		
 		break;
 	default:
+		encoded = NULL;
+		encoding = '\0';
 		g_assert_not_reached ();
 	}
 	
 	g_free (uword);
 	
-	g_string_sprintfa (string, "=?%s?%c?%s?=", charset, encoding, encoded);
+	g_string_append_printf (string, "=?%s?%c?%s?=", charset, encoding, encoded);
 }
 
 
